/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): Thanks to Helmut Denk (p7003ky@mail.lrz-muenchen.de)
 * for creating the CML AccountImplBean example.
 *
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.transaction.UserTransaction;
import javax.rmi.PortableRemoteObject;
import eb.Account;
import eb.AccountHome;

/**
 * @author Christophe Ney
 */

public class AccountServlet extends HttpServlet {

    private UserTransaction utx = null;

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
      PrintWriter out = response.getWriter();
      java.util.Date start = new java.util.Date();
      out.println("The time is " + start);
      response.setContentType("text/plain");
      Account a3 = null;
      try
      {
        // Default is Explicit Bean Managed Persistence (BMP)
	String beanName = "java:comp/env/ejb/HelloEnterpriseEntityExpl";
        String persistence = request.getParameter("persistence");
        if (persistence != null && persistence.equals("CMP"))
        {
          beanName = "java:comp/env/ejb/HelloEnterpriseEntityImpl";
          out.println("Using Container Managed Persistence (CMP)");
        } else {
          out.println("Using Bean Managed Persistence (BMP)");
        }
	// get JNDI initial context
	Context initialContext = null;
	try {
	    initialContext = new InitialContext();
	} catch (Exception e) {
	    out.println("Cannot get initial context for JNDI: " + e);
	    return;
	}

	// We want to start transactions from client: get UserTransaction
	out.println("Getting a UserTransaction object from JNDI");
	try {
	    utx = (UserTransaction)
		PortableRemoteObject.narrow(initialContext.lookup("java:comp/UserTransaction"),
				    UserTransaction.class);
	} catch (Exception e) {
	    out.println("Cannot lookup UserTransaction: "+e);
	    return;
	}

	// Connecting to Home thru JNDI
	out.println("Connecting to the AccountHome");
	AccountHome home = null;
	try {
	    home = (AccountHome)PortableRemoteObject.narrow(
	    					    initialContext.lookup(beanName),
	    					    AccountHome.class);
        } catch (Exception e) {
	    out.println("Cannot lookup "+beanName+": " +e);
	    return;
	}

	// List existing Accounts
	out.println("Getting the list of existing accounts in database");
	accountList(home,out);

	// Create a first Account
	out.println("Creating a new Account in database");
	try {
	    a3 = home.create(109, "Donald le Duck", 0);
	} catch (Exception e) {
	    out.println("Cannot create Account: " + e);
	    return;
	}

	// Find a second Account
	out.println("Finding two accounts by number (101, 102) in the database");
	Account a1 = null, a2 = null;
	try {
            a1 = home.findByPrimaryKey(new eb.AccountBeanPK(101));
	    a2 = home.findByNumber(102);
	} catch (Exception e) {
	    out.println("Cannot find Account: " + e);
	    return;
	}

	// First transaction (committed):
	// transfer 100 from a2 to a1
	out.println("Starting a first transaction, that will be committed");
	try {
	    long amount = 100;
            out.println("Transferring $" + amount + " from " + a2.printCustomer()
              + " to " + a1.printCustomer());
	    utx.begin();
	    a1.deposit(amount);
	    a2.withdraw(amount);
            utx.commit();
	} catch (Exception e) {
	    out.println("exception during 1st transaction: " + e);
            // Must make sure to roll-back if the transaction didn't complete
            // otherwise a nested transaction situation will occur.
            try
            {
	      utx.rollback();
            } catch (javax.transaction.SystemException x) {
              out.println("exception " + x + " during rollback of 1st transaction " + utx);
            }
	}

	// Start another transaction (rolled back):
	// transfer 20 from a1 to a2
	out.println("Starting a second transaction, that will be rolled back");
	try {
	    long amount = 20;
	    utx.begin();
            out.println("Transferring $" + amount + " from " + a1.printCustomer()
              + " to " + a2.printCustomer());
	    a1.withdraw(amount);
	    a2.deposit(amount);
	    utx.rollback();
	} catch (Exception e) {
	    out.println("exception during 2nd transaction: " + e);
            try
            {
              utx.rollback();
            } catch (javax.transaction.SystemException x) {
              out.println("Exception " + x + " during rollback of 2nd transaction " + utx);
            }
	}

	// List existing Accounts
	out.println("Getting the new list of accounts in database");
	accountList(home,out);

      } finally {
          if (a3!=null) {
            // Make sure we delete the account which was created, or there will be
                // duplicates!
            out.println("Removing Account for " + a3.printCustomer() + " previously created in database");
            try {
                a3.remove();
            } catch (Exception e) {
                out.println("exception during remove: " + e);
            }
          }
	// Exit program
	out.println("ClientAccount terminated");
        out.println("Elapsed time was " + (new java.util.Date().getTime() - start.getTime()) + " milliseconds");
      }
    }


    private void accountList(AccountHome h,PrintWriter out) {
	Enumeration alist;
	Account acc;
	try {
	    utx.begin();	// faster if made inside a Tx
	    alist = h.findAllAccounts();
	    while (alist.hasMoreElements()) {
		acc = (Account) alist.nextElement();
		out.println(acc.getNumber() + ":" + acc.printCustomer() + " " +acc.getBalance());
	    }
	    utx.commit();
	} catch (Exception e) {
	    out.println("Exception getting account list: " + e);
	    return;
	}
    }

}
