/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.Statement;

/**
 * @author Christophe Ney
 * a servlet that initialize the database
 */

public class InitServlet extends HttpServlet {

  public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException
  {
    PrintWriter out = response.getWriter();    response.setContentType("text/plain");

    java.util.Date start = new java.util.Date();
    out.println("The time is " + start);
    Context initialContext = null;
    try {
      initialContext = new InitialContext();
    } catch (Exception e) {
      out.println("Cannot get initial context for JNDI: " + e);
      return;
    }

    // lookup datasource
    DataSource ds = null;
    try {
      String dsName = "java:comp/env/jdbc/AccountDB";
      Object o = initialContext.lookup(dsName);
      ds = (DataSource)o;
      out.println("Successful lookup of DataSource: " + dsName);
    } catch (Exception e) {
      out.println( "Cannot lookup DataSource" + e);
      return;
    }

    // open a connection
    Connection conn = null;
    try {
      conn = ds.getConnection();
    } catch (Exception e) {
      java.io.StringWriter stackBuf = new java.io.StringWriter();
      e.printStackTrace(new PrintWriter(stackBuf));
      stackBuf.flush();
      out.println( "Cannot get connection: " + stackBuf);
      return;
    }

    // drop / create table
    // create table if not exist
    try {
      Statement st = conn.createStatement();
      out.println("drop table accountsample");
      st.execute("drop table accountsample");
      out.println("create table accountsample...");
      st.execute("create table accountsample (accno integer PRIMARY KEY ,customer VARCHAR(30), balance DECIMAL(15, 4))");
      st.execute("insert into accountsample values(101, 'Antoine de St Exupery', 200.00)");
      st.execute("insert into accountsample values(102, 'Alexandre Dumas Fils', 400.00)");
      st.execute("insert into accountsample values(103, 'Conan Doyle', 500.00)");
      st.execute("insert into accountsample values(104, 'Alfred de Musset', 100.00)");
      st.execute("insert into accountsample values(105, 'Phileas Lebegue', 350.00)");
      st.execute("insert into accountsample values(106, 'Alphonse de Lamartine', 650.00)");


      st.close();
      conn.close();

    } catch(Exception e){
      out.println( "Cannot drop/create table" + e);
      return;
    }

    out.println( "accountsample table created");
    out.println("Elapsed time was " + (new java.util.Date().getTime() - start.getTime()) + " milliseconds");

  }
}

