/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.transaction.Transaction;
import javax.transaction.UserTransaction;
import javax.transaction.TransactionManager;
import javax.rmi.PortableRemoteObject;

/**
 * A servlet that tests using the transaction service
 * 
 * @author Christophe Ney 
 */

public class JtaTestServlet extends HttpServlet {

  public void doGet(HttpServletRequest request, HttpServletResponse response)
  throws ServletException, IOException
  {
    PrintWriter out = response.getWriter();
    response.setContentType("text/plain");
    response.setHeader("Expires","0");

    out.print("new InitialContext...");
    Context initialContext = null;
    try {
      initialContext = new InitialContext();
    } catch (Exception e) {
      out.println("Cannot get initial context for JNDI: " + e);
      e.printStackTrace(out);
      return;
    }
    out.println("returned");

    out.print("lookup UserTransaction...");
    UserTransaction utx = null;
    try {
      utx = (UserTransaction) PortableRemoteObject.narrow(
        initialContext.lookup("java:comp/UserTransaction"),  UserTransaction.class);
    } catch (Exception e) {
      out.println("Cannot lookup UserTransaction: "+e);
      e.printStackTrace(out);
      return;
    }
    out.println("returned");

    try {
      out.print("Begin Transaction ...");
      utx.begin();
      out.println("Done");
      out.print("Commit Transaction ...");
      utx.commit();
      out.println("Done");
    } catch (Exception e) {
      out.println("exception during 1st Tx: " + e);
      e.printStackTrace(out);
      return;
    }
    out.print("lookup TransactionManager...");
    TransactionManager tm = null;
    try {
      tm = (TransactionManager)initialContext.lookup(
        "ems:/Services/Transaction/manager");
    } catch (Exception e) {
      out.println("Cannot lookup TransactionManager: "+e);
      e.printStackTrace(out);
      return;
    }
    out.println("returned");

    try {
      out.print("Begin Transaction ...");
      tm.begin();
      out.println("returned");
      out.print("get Transaction ...");
      Transaction tx = tm.getTransaction();
      out.println("returned");
      out.print("Commit Transaction ...");
      tm.commit();
      out.println("returned");
    } catch (Exception e) {
      out.println("exception with Transaction Manager: " + e);
      e.printStackTrace(out);
      return;
    }

    out.println("Test Completed");
  }


}

