/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.transaction.UserTransaction;
import javax.rmi.PortableRemoteObject;
import sb.Op;
import sb.OpHome;

/**
 * @author Christophe Ney
 */

public class OpServlet extends HttpServlet {

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        PrintWriter out = response.getWriter();
        response.setContentType("text/plain");

        java.util.Date start = new java.util.Date();
        out.println("The time is " + start);

	Context initialContext = null;
	try {
	    initialContext = new InitialContext();
	} catch (Exception e) {
	    out.println("Cannot get initial context for JNDI: " + e);
            e.printStackTrace(out);
	    return;
	}

	// We want to start transactions from client: get UserTransaction
	UserTransaction utx = null;
	try {
	    utx = (UserTransaction)PortableRemoteObject.narrow(
                initialContext.lookup("java:comp/UserTransaction"),
				    UserTransaction.class);
	} catch (Exception e) {
	    out.println("Cannot lookup UserTransaction: "+e);
            e.printStackTrace(out);
	    return;
	}

	// Connecting to OpHome thru JNDI
	OpHome home = null;
	try {
	    home = (OpHome)PortableRemoteObject.narrow(
                initialContext.lookup("java:comp/env/ejb/HelloEnterpriseSession"), 
                                      OpHome.class);
	} catch (Exception e) {
	    out.println( "Cannot lookup OpHome: " + e);
            e.printStackTrace(out);
	    return;
	}

	// OpBean creation
	Op order = null;
	try {
	    out.println("Create a Session Bean proxy (OpBean)");
	    order = home.create("User1");
	} catch (Exception e) {
	    out.println("Cannot create Session Bean proxy (OpBean): " + e);
        e.printStackTrace(out);
	    return;
	}
	
    // checking Principal propagation
    // create a valid principal
    class MyPrincipal implements java.io.Serializable,java.security.Principal {
        public String getName() {
            return "Christophe Ney";
        }
    }
    MyPrincipal pr = new MyPrincipal();
    // create a subject with the principal
    org.enhydra.security.auth.Subject subject = new org.enhydra.security.auth.Subject();
    subject.getPrincipals().add(pr);
    
    class MyPrivilegedExceptionAction implements java.security.PrivilegedExceptionAction {
            private Op order;

            public MyPrivilegedExceptionAction(Op order) {
                this.order = order;
            }
            
            public Object run() throws Exception {
                return order.getPrincipalName();
            }
    }
     
    // invoke ejb with the principal
    try {
        out.println("Calling Session bean with Principal "+pr.getName());
        Object ret= org.enhydra.security.auth.Subject.doAsPrivileged(subject,new MyPrivilegedExceptionAction(order),null);
        out.println("CallerPrincipal in Session bean is "+ret.toString());
    } catch (java.security.PrivilegedActionException e) {
	    out.println("Cannot get CallerPrincipal" + e);
        e.printStackTrace(out);
    }
	// First transaction (committed)
	try {
	    out.println("Start a first transaction");
	    utx.begin();
	    out.println("First request on the new bean: purchase 10 shares");
	    order.buy(10);
	    out.println("Second request on the bean: purchase 20 shares");
	    order.buy(20);
	    out.println("Commit the transaction");
	    utx.commit();
	} catch (Exception e) {
	    out.println("exception during 1st Tx: " + e);
            e.printStackTrace(out);
	    return;
	}
        out.println("The order contains " + order.read() + " shares");
	// Start another transaction (rolled back)
	try {
	    out.println("Start a second transaction: purchase 50 shares");
	    utx.begin();
	    order.buy(50);
	    out.println("Rollback the transaction");
	    utx.rollback();
	} catch (Exception e) {
	    out.println("exception during 2nd Tx: " + e);
            e.printStackTrace(out);
	    return;
	}

	// Get the total bought, outside the transaction
	int val = 0;
	try {
	    out.println("Request outside any transaction");
	    val = order.read();
	} catch (Exception e) {
	    out.println("Cannot read value on order : "+e);
            e.printStackTrace(out);
	    return;
	}
        out.println("The order contains " + order.read() + " shares");
	if (val != 10+20) {
	    out.println("Bad value read: "+val);
	    return;
	}

	// Remove Session bean
	try {
	    order.remove();
	} catch (Exception e) {
	    out.println("Exception on buy: "+e);
            e.printStackTrace(out);
	    return;
	}
	out.println("ClientOp OK. Exiting.");
        out.println("Elapsed time was " + (new java.util.Date().getTime() - start.getTime()) + " milliseconds");
    }
}

